// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

// __Please ask for permission, if you want to use some code from this shader__

// __Discord: The Alexander # 3744__

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		in vec2 uv0;
		in vec2 uv1;
	#endif
#endif

#include "uniformPerFrameConstants.h"
#include "uniformShaderConstants.h"
#include "util.h"
#include "lib/Hash.h"
uniform highp float TOTAL_REAL_WORLD_TIME;

varying highp vec3 cpos;
varying highp vec3 wpos;
varying vec4 color;
varying float farrender;
varying float farchunk;

#ifdef UNDERWATER
varying float fogr;
#endif

/////////////////////////////
/////////// STOP! ///////////
// Code water by: @bambo_san
// Please, if you want to use the water code, ask @bambo_san for   
// permission, it does not cost anything
// Att: @The_AlexanderG
//////////////////////////////
//////////////////////////////

#define rain (1.-pow(FOG_CONTROL.y,10.))*uv1.y

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

float calcWave(highp vec2 pos){
	highp vec2 htime = vec2(0.,TOTAL_REAL_WORLD_TIME);
		pos *= 40.;
	highp vec2 wp = pos * mat2(vec2(sin(1.),cos(1.8)),vec2(sin(1.),cos(1.)))+(htime * 70.) * 4.0; wp.y *= 2.0;
	highp vec2 wp1 = pos * mat2(vec2(sin(1.2),cos(0.5)),vec2(sin(.5),cos(0.5)))+(htime * 70.) * 1.5; wp1.y *= 1.0;

	highp float wave = -noise(wp * 0.005) * 10.0;
		wave += noise(wp1 * 0.01) * 10.0; wave *= 0.2;
return wave;
}

vec3 skycolor(in float dfog, in float nfog, in float lenpos){
	vec3 skycolor = mix(vec3(.0,.4,.8),vec3(.2,.4,.9)*.34,nfog);
		skycolor = mix(skycolor,vec3(0,0.2,0.6),dfog);
		skycolor = mix(skycolor,FOG_COLOR.rgb*1.5,rain);

	vec3 scattercolor = mix(FOG_COLOR.rgb,FOG_COLOR.rgb+vec3(FOG_COLOR.r,FOG_COLOR.g,FOG_COLOR.b*2.)*1.2,dfog*3.);
		scattercolor = mix(scattercolor,FOG_COLOR.rgb*1.3,rain);
return mix(skycolor,scattercolor,pow(lenpos,5.));
}

vec3 calclight(vec3 c,float strength){
	vec3 lcolor = mix(vec3(dot(vec3(7.,3.,0)*strength,vec3(1.))),vec3(7.,3.,0)*strength,.9);
	c += lcolor*pow(1.1*uv1.x,5.);
return c;
}

vec4 calcWater(in vec4 btex, in vec3 N, in vec2 ttime, in float dfog, in float nfog){
	float rim = smoothstep(0., 1., abs(1. - dot(N, normalize(-wpos))))*uv1.y, waterw = calcWave(cpos.xz*3.);
	vec3 nw = normalize(vec3(waterw*.5,-waterw*.5,1.));
	vec4 wcolor = vec4(0.1,.1,.1,.5);

	highp vec3 spos = wpos;
	if(spos.x > 0.){ spos -= spos.x*.45; } else { spos += spos.x*.45; }
        spos = normalize(spos * vec3(1,1,6));
        spos.xy += nw.xy*.5;
	float sm = (1.-rain)*smoothstep(1.,0.,length(spos.zy));

    highp vec3 rv = reflect(normalize(-wpos), N);
       rv /= rv.y - .5;
	 highp float hmap = pow(dot(nw,vec3(.8,1.,.8)),5.);
    highp float dvalue = mix(.03,.1,ttime.x);

	   wcolor.rgb = mix(wcolor.rgb,calclight(wcolor.rgb, dvalue),uv1.x+pow(hmap,3.)*10.);
       wcolor = mix(wcolor+hmap*.1,vec4(skycolor(dfog,nfog,rim),1.),rim+hmap*.1);
       wcolor += (dfog*4.)*sm*smoothstep(0.,.8,uv1.y)*pow(rim,2.)*btex.r;
return wcolor;
}

vec3 calcFog(in vec3 c,in vec2 ttime, in float dfog, in float nfog){
	highp vec3 spos = wpos;
	if(spos.x > 0.){ spos -= spos.x*.45; } else { spos += spos.x*.45; }
		spos = normalize(spos);
	float sunamount = smoothstep(1.,0.,length(spos.zy));
	vec3 fogamount = mix(vec3(0.7,0.85,1.)*FOG_COLOR.rgb,FOG_COLOR.rgb+vec3(FOG_COLOR.r,FOG_COLOR.g,FOG_COLOR.b*2.),sunamount*(3.*dfog));
    c = mix(c,fogamount,pow(farrender,3.)*.7);
    c = mix(c,fogamount,clamp(farrender*1.5,0.,.9)*ttime.y);
    c = mix(c,FOG_COLOR.rgb*1.2,clamp(farrender*2.,0.,1.)*rain);
return c;
}

//https://knarkowicz.wordpress.com/2016/01/06/aces-filmic-tone-mapping-curve/
vec3 acesFilmic(vec3 x){
    const float a = 2.51;
    const float b = 0.035;
    const float c = 2.43;
    const float d = 0.59;
    const float e = 0.15;
return clamp((x * (a * x + b)) / (x * (c * x + d) + e), 0.0, 1.0);
}

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else
#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
     vec4 lm = texture2D_AA(TEXTURE_1, vec2(0,1));
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
	vec4 lm = texture2D(TEXTURE_1, vec2(0,1));
#endif

	highp vec2 ttime = vec2(max(1.-min(pow(lm.r*1.8,5.),1.),.0),pow(clamp((1.-pow(lm.r,5.))*1.3,.0,1.),8.))*uv1.y;
    float dfog = (1.-FOG_COLOR.b)*clamp((FOG_COLOR.r-.15)*1.25,0.,1.), nfog = pow(max(min(1.0-FOG_COLOR.r*1.5,1.),.0),1.2);

#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

	vec4 inColor = color;

bool nether = ((FOG_COLOR.r > FOG_COLOR.b) && (FOG_COLOR.r < 0.5) && (FOG_COLOR.b < 0.2));
float cave = 1.0-uv1.y;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

	vec4 btex = texture2D(TEXTURE_1,vec2(pow(uv1.x,2.), uv1.y));

#if !defined(ALWAYS_LIT)
	diffuse *= btex;
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif

//reduce default ambient occlusion
if(color.g * 2.0 > color.b + color.r && color.a != 0.){
	diffuse.rgb *= mix(normalize(inColor.rgb),inColor.rgb,0.5);
}else{
	diffuse.rgb *= color.a ==0.?inColor.rgb: sqrt(inColor.rgb);
}

#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

	vec3 N = normalize(cross(dFdx(cpos.xyz), dFdy(cpos.xyz)));
    highp float smap = 0.;
if(color.a == 0.){
	smap = mix(smap, 1., clamp(abs(-N.x), 0., 1.)); smap = mix(smap, 1., clamp((-N.z + N.y), 0., 1.) * ttime.y*(1.-ttime.x));
} else {
   #if !USE_ALPHA_TEST
       smap = mix(smap, 1., clamp(abs(-N.x), 0., 1.)); smap = mix(smap, 1., clamp((-N.z + N.y), 0., 1.) * ttime.y*(1.- ttime.x));
   #endif
}
    highp float ub = smoothstep(.87, .845, uv1.y);
    smap = mix(smap, 1., ub);smap = mix(smap,0.,rain);smap = mix(smap, 0., smoothstep(lm.r*uv1.y, 1., uv1.x));

	vec3 exposureBias = mix(mix(mix(vec3(2.,2.,2.),vec3(2.5,2.2,1.),ttime.y),vec3(2.,2.2,2.4),ttime.x),vec3(.7,.8,.9),rain);
   diffuse.rgb *= pow(diffuse.rgb,(color.a==0.)?vec3(.5,.5,.5):vec3(.6,.6,.6));
	vec3 curr = acesFilmic(exposureBias*diffuse.rgb);
	diffuse.rgb = curr;

    bool waterd = (color.g < color.b && color.g > color.r);

//torch
if(!waterd==false){
vec3 colorA = vec3(0.8,0.45,0.0);
vec3 colorB = vec3(0.9,0.40,0.0);
highp float ti = abs(sin(TIME));
vec3 light = mix(colorA,colorB,ti);
diffuse.rgb += light *max(uv1.x-0.45,0.0)*(1.0-diffuse.rgb);

}

#if !defined(SEASONS) && !defined(ALPHA_TEST)
if(waterd){diffuse = calcWater(btex,N,ttime,dfog,nfog);}
#endif

	diffuse.rgb *= mix(1.+.3*(N.z)*ttime.y*(1.-ttime.x),.4,smap);
	highp float dvalue= mix(mix(mix(mix(.08,0.,uv1.y),.03,ttime.y),.08,ttime.x),.08,rain);
	diffuse.rgb = calclight(diffuse.rgb, dvalue);

#ifdef UNDERWATER
if(!waterd){diffuse = vec4(.3,.7,1.,.0)*diffuse + max(0.,calcWave(cpos.xz*3.))*diffuse*uv1.y;}
	diffuse.rgb += (1.-uv1.y)*pow(uv1.x*1.1,4.)*sqrt(diffuse.rgb);
	diffuse.rgb = mix(diffuse.rgb,FOG_COLOR.rgb,pow(fogr,10.));
#endif

if(nether==true){

vec3 nether_fog = mix(diffuse.rgb, FOG_COLOR.rgb*vec3(1.5,0.5,0.0), clamp(length(-wpos)/RENDER_DISTANCE*1.0,0.0,0.6));

diffuse.rgb = nether_fog;
}

//secondary fog
	diffuse.rgb = calcFog(diffuse.rgb, ttime, dfog, nfog);

//out
	gl_FragColor = diffuse;
#endif // BYPASS_PIXEL_SHADER
}
